@@ -1,3 +1,4 @@
+from os import urandom
 import numpy as np
 import math
 import base64
@@ -304,20 +305,7 @@ class PyImpact:
 
     For a general guide on impact sounds in TDW, read [this](../misc_frontend/impact_sounds.md).
 
-    Usage:
-
-    ```python
-    from tdw.controller import Controller
-    from tdw.py_impact import PyImpact
-
-    p = PyImpact()
-    c = Controller()
-    c.start()
-
-    # Your code here.
-
-    c.communicate(p.get_impact_sound_command(arg1, arg2, ... ))
-    ```
+    For example usage, see: `tdw/Python/example_controllers/impact_sounds.py`
     """
 
     def __init__(self, initial_amp: float = 0.5, prevent_distortion: bool = True, logging: bool = False):
@@ -347,6 +335,116 @@ class PyImpact:
         # Create empty dictionary for log.
         self.mode_properties_log = dict()
 
+        # Get the default object info. See: `set_default_audio_info()`
+        self.object_info: Dict[str, ObjectInfo] = dict()
+        # A dictionary of objects in the scene. See: `set_default_audio_info()`
+        self.object_names: Dict[int, str] = dict()
+        # A dummy ID for the environment. See: `set_default_audio_info()`
+        self.env_id: int = -1
+
+    def set_default_audio_info(self, object_names: Dict[int, str]) -> None:
+        """
+        Set the default audio info for each object in the scene.
+
+        :param object_names: A dictionary of objects in the scene. Key = The object ID. Value = The name of the model.
+        """
+
+        self.object_names = object_names
+        self.object_info = PyImpact.get_object_info()
+        self.env_id = int.from_bytes(urandom(3), byteorder='big')
+
+    def get_audio_commands(self, resp: List[bytes], floor: AudioMaterial, wall: AudioMaterial, resonance_audio: bool = False) -> List[dict]:
+        """
+        Given the response from the build, get audio commands.
+
+        Required:
+
+        - You've already called `set_default_audio_info()` and supplied it with the IDs and names of each object in the scene:
+        - You've requested collision and rigidbody data per-frame (`send_collisions` and `send_rigidbodies`)
+
+        :param resp: The response from the build.
+        :param floor: The floor material as an `AudioMaterial` enum value.
+        :param wall: The wall material as an `AudioMaterial` enum value.
+        :param resonance_audio: If True, this function assumes you're using Resonance Audio.
+
+        :return: A list of commands that will play audio generated by the collisions on this frame. This can be empty.
+        """
+
+        commands = []
+        collisions, env_collisions, rigidbodies = self.get_collisions(resp=resp)
+        # Get the mass and speed of each object.
+        speeds: Dict[int, float] = dict()
+        for i in range(rigidbodies.get_num()):
+            object_id = rigidbodies.get_id(i)
+            speeds[object_id] = np.linalg.norm(rigidbodies.get_velocity(i))
+        # Get all stays and exits.
+        stays_and_exits: List[int] = list()
+        for c in collisions:
+            if c.get_state() == "stay" or c.get_state() == "exit":
+                stays_and_exits.extend([c.get_collider_id(), c.get_collidee_id()])
+        colliders: List[int] = list()
+        # Play sounds from collisions.
+        for collision in collisions:
+            # Ignore invalid collisions.
+            if collision.get_state() != "enter" or not self.is_valid_collision(collision=collision):
+                continue
+            collider_id = collision.get_collider_id()
+            collidee_id = collision.get_collidee_id()
+            # Ignore collisions if there is a "stay" event, because this can create a droning effect.
+            if collider_id in stays_and_exits or collidee_id in stays_and_exits or\
+                    collider_id in colliders or collidee_id in colliders:
+                continue
+            colliders.append(collider_id)
+            colliders.append(collidee_id)
+            # Skip objects that for some reason aren't in the cached data.
+            if collider_id not in self.object_names or collidee_id not in self.object_names:
+                continue
+            # The target object is the one with less mass.
+            if self.object_info[self.object_names[collider_id]].mass < \
+                    self.object_info[self.object_names[collidee_id]].mass:
+                target = collider_id
+                other = collidee_id
+            else:
+                target = collidee_id
+                other = collider_id
+
+            target_audio = self.object_info[self.object_names[target]]
+            other_audio = self.object_info[self.object_names[other]]
+            commands.append(self.get_impact_sound_command(collision=collision,
+                                                          rigidbodies=rigidbodies,
+                                                          target_id=target,
+                                                          target_amp=target_audio.amp,
+                                                          target_mat=target_audio.material.name,
+                                                          other_id=other,
+                                                          other_amp=other_audio.amp,
+                                                          other_mat=other_audio.material.name,
+                                                          resonance=target_audio.resonance,
+                                                          play_audio_data=not resonance_audio))
+        # Play sounds from collisions with the environment.
+        stays_and_exits.extend([c.get_object_id() for c in env_collisions if
+                                c.get_state() == "stay" or c.get_state() == "exit"])
+        for collision in env_collisions:
+            target = collision.get_object_id()
+            # Ignore collisions that aren't enter, not a floor, that aren't in the cached data, or that are too slow.
+            # When objects are initially spawned they collide with the environment at very slow speeds,
+            # resulting in a "click" sound that we don't actually want.
+            if collision.get_state() != "enter" or not collision.get_floor() or target not in self.object_names or \
+                    speeds[target] < 0.01 or target in colliders or target in stays_and_exits:
+                continue
+            colliders.append(target)
+            audio = self.object_info[self.object_names[target]]
+            commands.append(self.get_impact_sound_command(collision=collision,
+                                                          rigidbodies=rigidbodies,
+                                                          target_id=target,
+                                                          target_amp=audio.amp,
+                                                          target_mat=audio.material.name,
+                                                          other_id=self.env_id,
+                                                          other_amp=0.5,
+                                                          other_mat=floor.name,
+                                                          resonance=audio.resonance,
+                                                          play_audio_data=not resonance_audio))
+        return commands
+
     def get_log(self) -> dict:
         """
         :return: The mode properties log.
@@ -422,6 +520,9 @@ class PyImpact:
             for i in range(rigidbodies.get_num()):
                 if rigidbodies.get_id(i) == id2:
                     vel = rigidbodies.get_velocity(i)
+                    # If the y coordinate of the velocity is negative, it implies a scrape or roll along the floor.
+                    if vel[1] < 0:
+                        return None
                     break
         vel = np.asarray(vel)
         speed = np.square(vel)
@@ -438,7 +539,7 @@ class PyImpact:
             tmp = speed * np.cos(tmp)
             nspd.append(tmp)
         normal_speed = np.mean(nspd)
-        # Get indices of objects in collisions
+        # Get indices of objects in collisions.
         id1_index = None
         id2_index = None
 
@@ -448,12 +549,22 @@ class PyImpact:
             if rigidbodies.get_id(i) == id2:
                 id2_index = i
 
-        # Make sure both IDs were found. If they aren't, don't return a sound.
-        if obj_col and (id1_index is None or id2_index is None):
+        # Use default values for environment collisions.
+        if not obj_col:
+            m1 = 100
+            m2 = rigidbodies.get_mass(id2_index)
+        # Use the Rigidbody masses.
+        elif id1_index is not None and id2_index is not None:
+            m1 = rigidbodies.get_mass(id1_index)
+            m2 = rigidbodies.get_mass(id2_index)
+        # Fallback: Try to use default mass values if the ID's aren't in the Rigidbody data.
+        elif id1 in self.object_names and id2 in self.object_names and self.object_names[id1] in self.object_info and \
+                self.object_names[id2] in self.object_info:
+            m1 = self.object_info[self.object_names[id1]].mass
+            m2 = self.object_info[self.object_names[id2]].mass
+        # Failed to generate a sound.
+        else:
             return None
-
-        m1 = rigidbodies.get_mass(id1_index) if obj_col else 1000
-        m2 = rigidbodies.get_mass(id2_index)
         mass = np.min([m1, m2])
 
         # Re-scale the amplitude.
@@ -489,7 +600,7 @@ class PyImpact:
         self.object_modes[id2][id1].count_collisions()
 
         # Prevent distortion by clamping the amp.
-        if self.prevent_distortion and amp > 0.99:
+        if self.prevent_distortion and np.abs(amp) > 0.99:
             amp = 0.99
 
         sound = amp * sound / np.max(np.abs(sound))
